@@ -5,7 +5,6 @@ import re
 import misopy
 from misopy.gff_utils import *
 from misopy.parse_csv import *
-from misopy.json_utils import *
 import pprint
 
 
@@ -283,10 +282,6 @@ class Gene:
 	# ensure that the parts the coordinates map to are in the isoform
 	start_part = self.get_part_by_coord(genomic_start)
 	end_part = self.get_part_by_coord(genomic_end)
-
-#        print "COORDS2ISO: genomic_start, genomic_end: ", genomic_start, \
-#              genomic_end
-#        print "--->",start_part, end_part
         
 	if start_part == None or end_part == None:
 	    return None, None
@@ -870,12 +865,17 @@ def print_gene_hierarchy(gene_hierarchy):
 #    pp = pprint.PrettyPrinter(indent=4)
 #    pp.pprint(gene_hierarchy)
 
-def load_genes_from_gff(gff_filename):
+def load_genes_from_gff(gff_filename,
+                        include_introns=False,
+                        reverse_recs=False,
+                        suppress_warnings=False):
     """
     Load all records for a set of genes from a given GFF file.
     Parse each gene into a Gene object.
     """
-    gff_db = GFFDatabase(gff_filename)
+    gff_db = GFFDatabase(gff_filename,
+                         include_introns=include_introns,
+                         reverse_recs=reverse_recs)
     # dictionary mapping gene IDs to the list of all their relevant records
     gff_genes = {}
 
@@ -888,7 +888,8 @@ def load_genes_from_gff(gff_filename):
         gene_label = gene.get_id()
 
         if gene_label not in gene_hierarchy:
-            print "Skipping gene %s..." %(gene_label)
+            if not suppress_warnings:
+                print "Skipping gene %s..." %(gene_label)
             continue
         
         gene_hierarchy[gene_label]['gene'] = gene
@@ -898,17 +899,20 @@ def load_genes_from_gff(gff_filename):
                                               gene_hierarchy[gene_label],
                                               gene_records)
         if gene_obj == None:
-            print "Cannot make gene out of %s" %(gene_label)
+            if not suppress_warnings:
+                print "Cannot make gene out of %s" %(gene_label)
             continue
         gff_genes[gene.get_id()] = {'gene_object': gene_obj,
                                     'hierarchy': gene_hierarchy}
 
         if (num_genes % 5000) == 0:
-            print "Through %d genes..." %(num_genes)
+            if not suppress_warnings:
+                print "Through %d genes..." %(num_genes)
         num_genes += 1
 
     num_genes = len(gff_genes)
-    print "Loaded %d genes" %(num_genes)
+    if not suppress_warnings:
+        print "Loaded %d genes" %(num_genes)
 
     return gff_genes
 
@@ -990,10 +994,10 @@ def make_gene_from_gff_records(gene_label,
         # Record transcript ids that are not skipped
         used_transcript_ids.append(transcript_id)
 
-    if num_transcripts_with_exons < 2:
-        print "WARNING: %s does not have at least two mRNA/transcript entries " \
-              "with exons. Skipping over..." %(gene_label)
-        return None
+    #if num_transcripts_with_exons < 2:
+    #    print "WARNING: %s does not have at least two mRNA/transcript entries " \
+    #          "with exons. Skipping over..." %(gene_label)
+    #    return None
 
     # Compile all exons used in all transcripts
     all_exons = []
