@@ -14,6 +14,7 @@ from collections import defaultdict
 
 import misopy
 import misopy.gff_utils as gff_utils
+import misopy.misc_utils as misc_utils
 
 def is_exon_in_mRNA(gff_in, exon, mRNA):
     """
@@ -167,6 +168,9 @@ def map_bam2gff(bam_filename, gff_filename,
     # in the output BAM file. Thanks to Aaron Quinlan for implementing
     # this helpful feature.
     print "Preparing to call bedtools \'tagBam\'"
+    if misc_utils.which("tagBam") is None:
+        print "Aborting operation.."
+        sys.exit(1)
     tagBam_cmd = get_tagBam_cmd(bam_filename, interval_label,
                                 gff_filename)
     # Write intervals as BAM
@@ -286,24 +290,26 @@ def get_const_exons_by_gene(gff_filename, output_dir,
 
     for gene, mRNAs in gff_in.mRNAs_by_gene.iteritems():
         # For each gene, look at all mRNAs and return constitutive exon
-        curr_const_exons = get_const_exons_from_mRNA(gff_in, mRNAs,
-                                                     all_constitutive=all_constitutive,
-                                                     min_size=min_size)
+        curr_const_exons = \
+            get_const_exons_from_mRNA(gff_in, mRNAs,
+                                      all_constitutive=all_constitutive,
+                                      min_size=min_size)
         const_exons_by_gene.extend(curr_const_exons)
         num_exons += len(curr_const_exons)
 
     t2 = time.time()
+
+    basename = re.sub("[.]gff3?", "", os.path.basename(gff_filename))
     if output_filename is None:
         # Create default output filename if not
         # given one as argument
         output_filename = os.path.join(output_dir,
                                        "%s.min_%d.const_exons.gff" \
-                                       %(os.path.basename(gff_filename).replace(".gff", ""),
+                                       %(basename,
                                          min_size))
     if not all_constitutive:
         print "Constitutive exon retrieval took %.2f seconds (%d exons)." \
               %((t2 - t1), num_exons)
-
         output_exons_to_file(const_exons_by_gene, output_filename,
                              output_format=output_format)
     else:
@@ -312,29 +318,46 @@ def get_const_exons_by_gene(gff_filename, output_dir,
     return const_exons_by_gene, output_filename
 
 
+def greeting():
+    print "Utility for fetching constitutive exons from "
+    print "GFF files. Optionally fetch constitutive exons by size."
+    print "Usage:\n"
+    print "To fetch constitutive exons from GFF: "
+    print "  exon_utils.py --get-const-exons input.gff --output-dir outdir\n"
+    print "See --help for more options."
+
+
 def main():
     from optparse import OptionParser
     parser = OptionParser()
-    parser.add_option("--get-const-exons", dest="get_const_exons", nargs=1, default=None,
+    parser.add_option("--get-const-exons", dest="get_const_exons",
+                      nargs=1, default=None,
                       help="Get constitutive exons from an input GFF file.")
-    parser.add_option("--min-exon-size", dest="min_exon_size", nargs=1, type="int", default=20,
-                      help="Minimum size of constitutive exon (in nucleotides) that should be used "
-                      "in the computation. Default is 20 bp.")
-    parser.add_option("--output-dir", dest="output_dir", nargs=1, default=None,
+    parser.add_option("--min-exon-size", dest="min_exon_size",
+                      nargs=1, type="int", default=20,
+                      help="Minimum size of constitutive exon (in nucleotides) "
+                      "that should be used in the computation. "
+                      "Default is 20 bp.")
+    parser.add_option("--output-dir", dest="output_dir",
+                      nargs=1, default=None,
                       help="Output directory.")
     (options, args) = parser.parse_args()
 
     if options.output_dir == None:
+        greeting()
         print "Error: need --output-dir."
         return
         
     output_dir = os.path.abspath(os.path.expanduser(options.output_dir))
 
     if options.get_const_exons != None:
-        gff_filename = os.path.abspath(os.path.expanduser(options.get_const_exons))
+        gff_filename = \
+            os.path.abspath(os.path.expanduser(options.get_const_exons))
         get_const_exons_by_gene(gff_filename,
                                 output_dir,
                                 min_size=options.min_exon_size)
+    else:
+        greeting()
         
 
 if __name__ == '__main__':
