@@ -98,13 +98,15 @@ def compute_prior_proportion_diff(num_samples):
     return array(samples)
 
 
-
-def compute_delta_densities(samples1_filename, samples2_filename, diff_range,
+def compute_delta_densities(samples1_filename,
+                            samples2_filename,
+                            diff_range,
                             smoothing_param=0.3):
     """
-    Compute the Gaussian kernel density fitted distributions over delta for
-    the two sets of posterior samples filenames given.  Returns the posterior density
-    and prior density as well, assuming a uniform prior over the Psi of the samples
+    Compute the Gaussian kernel density fitted distributions
+    over delta for the two sets of posterior samples filenames given.
+    Returns the posterior density and prior density as well, assuming
+    a uniform prior over the Psi of the samples
     in the two conditions.
     """
     densities = {}
@@ -149,6 +151,11 @@ def compute_delta_densities(samples1_filename, samples2_filename, diff_range,
     warning_outputted = False
     
     for iso_num in range(num_isoforms):
+        ##
+        ## TODO: If distinct number of samples are used
+        ## in the two samples compared, take the minimum
+        ## number of samples
+        ##
         posterior_diff = posterior_samples1[:, iso_num] - \
                          posterior_samples2[:, iso_num]
 
@@ -171,8 +178,9 @@ def compute_delta_densities(samples1_filename, samples2_filename, diff_range,
             posterior_density = gaussian_kde_covfact(posterior_diff,
                                                      smoothing_param)
 
-        bayes_factor, diff_prior, diff_posterior = compute_bayes_factor(analytic_prior_density,
-                                                                        posterior_density)
+        bayes_factor, diff_prior, diff_posterior = \
+            compute_bayes_factor(analytic_prior_density,
+                                 posterior_density)
         # Edge case: if improperly samples, maintain Bayes factors
         # as list to be consistent with multi-isoform case
         densities['bayes_factor'].append(bayes_factor)
@@ -223,11 +231,6 @@ def output_samples_comparison(sample1_dir, sample2_dir, output_dir,
     bf_output_dir = os.path.join(output_dir, 'bayes-factors/')
     if not os.path.isdir(bf_output_dir):
 	os.mkdir(bf_output_dir)
-	
-    # Create directory for raw delta posteriors
-    dp_output_dir = os.path.join(output_dir, 'delta-posteriors/')
-    if not os.path.isdir(dp_output_dir):
-	os.makedirs(dp_output_dir)
     
     header_fields = ['event_name',
 		     'sample1_posterior_mean',
@@ -248,29 +251,28 @@ def output_samples_comparison(sample1_dir, sample2_dir, output_dir,
                      'mRNA_starts',
                      'mRNA_ends']
     header_line = "\t".join(header_fields) + "\n"
-    output_filename = os.path.join(bf_output_dir, "%s_vs_%s.miso_bf" %(sample1_label,
-                                                                       sample2_label))
+    output_filename = \
+        os.path.join(bf_output_dir, "%s_vs_%s.miso_bf" %(sample1_label,
+                                                         sample2_label))
     output_file = open(output_filename, 'w')
     output_file.write(header_line)
 
     num_events_compared = 0
-
-    # Number of events to put into directories -- used to
-    # split up the raw delta-posteriors
-    batch_size = 500
-
+    
     file_num = 0
-    curr_batch = file_num
 
     compressed_ids_to_genes = None
     if use_compressed is not None:
-        print "  - Loading compressed IDs mapping from: %s" %(use_compressed)        
-        compressed_ids_to_genes = index_gff.load_compressed_ids_to_genes(use_compressed)
+        print "  - Loading compressed IDs mapping from: %s" \
+              %(use_compressed)        
+        compressed_ids_to_genes = \
+            index_gff.load_compressed_ids_to_genes(use_compressed)
 
     # Compute the Bayes factors for each file
     for sample1_filename in sample1_filenames:
-        sample1_event_name = get_event_name(sample1_filename,
-                                            use_compressed_map=compressed_ids_to_genes)
+        sample1_event_name = \
+            get_event_name(sample1_filename,
+                           use_compressed_map=compressed_ids_to_genes)
 
         # Parameters from raw MISO samples file
         params = parse_sampler_params(sample1_filename)
@@ -278,9 +280,10 @@ def output_samples_comparison(sample1_dir, sample2_dir, output_dir,
         gene_info = get_gene_info_from_params(params)
         
 	# Find corresponding event filename in sample 2
-        sample2_filename = filter(lambda filename:
-                                  get_event_name(filename,
-                                                 use_compressed_map=compressed_ids_to_genes) == sample1_event_name,
+        sample2_filename = \
+            filter(lambda filename:
+                   get_event_name(filename,
+                                  use_compressed_map=compressed_ids_to_genes) == sample1_event_name,
                                   sample2_filenames)
 	if len(sample2_filename) == 0:
             continue
@@ -290,8 +293,10 @@ def output_samples_comparison(sample1_dir, sample2_dir, output_dir,
 	
 	# Compute delta of posterior samples and Bayes factors
 	diff_range = arange(-1, 1, 0.001)
-	delta_densities = compute_delta_densities(sample1_filename, sample2_filename,
-                                                  diff_range)
+	delta_densities = \
+            compute_delta_densities(sample1_filename,
+                                    sample2_filename,
+                                    diff_range)
 	bf = delta_densities['bayes_factor']
         
         num_isoforms = shape(delta_densities['samples1'])[1]
@@ -299,7 +304,6 @@ def output_samples_comparison(sample1_dir, sample2_dir, output_dir,
 	sample1_posterior_mean = mean(delta_densities['samples1'], 0)
 	sample2_posterior_mean = mean(delta_densities['samples2'], 0)
 
-
         # Get the labels of the isoforms
         isoforms_field = delta_densities['isoforms']
 
@@ -308,16 +312,18 @@ def output_samples_comparison(sample1_dir, sample2_dir, output_dir,
         sample2_counts_info = delta_densities['sample2_counts']
 
 	# Compute posterior mean and credible intervals for sample 1
-        sample1_cred_intervals = format_credible_intervals(sample1_event_name,
-                                                           delta_densities['samples1'],
-                                                           confidence_level=alpha)
+        sample1_cred_intervals = \
+            format_credible_intervals(sample1_event_name,
+                                      delta_densities['samples1'],
+                                      confidence_level=alpha)
         sample1_ci_low = sample1_cred_intervals[2]
         sample1_ci_high = sample1_cred_intervals[3]
 
         # Compute posterior mean and credible intervals for sample 2
-        sample2_cred_intervals = format_credible_intervals(sample1_event_name,
-                                                           delta_densities['samples2'],
-                                                           confidence_level=alpha)
+        sample2_cred_intervals = \
+            format_credible_intervals(sample1_event_name,
+                                      delta_densities['samples2'],
+                                      confidence_level=alpha)
         sample2_ci_low = sample2_cred_intervals[2]
         sample2_ci_high = sample2_cred_intervals[3]
         
@@ -325,12 +331,15 @@ def output_samples_comparison(sample1_dir, sample2_dir, output_dir,
 
 	# Use precision of two decimal places
         if num_isoforms == 2:
-            sample1_posterior_mean = Decimal(str(sample1_posterior_mean[0])).quantize(Decimal('0.01'))
-            sample2_posterior_mean = Decimal(str(sample2_posterior_mean[0])).quantize(Decimal('0.01'))
+            sample1_posterior_mean = \
+                Decimal(str(sample1_posterior_mean[0])).quantize(Decimal('0.01'))
+            sample2_posterior_mean = \
+                Decimal(str(sample2_posterior_mean[0])).quantize(Decimal('0.01'))
             posterior_diff = "%.2f" %(sample1_posterior_mean - sample2_posterior_mean)
             bayes_factor = "%.2f" %(bf[0])
         else:
-            posterior_diff = ",".join(["%.2f" %(v) for v in (sample1_posterior_mean - sample2_posterior_mean)])
+            posterior_diff = \
+                ",".join(["%.2f" %(v) for v in (sample1_posterior_mean - sample2_posterior_mean)])
             sample1_posterior_mean = sample1_cred_intervals[1]
             sample2_posterior_mean = sample2_cred_intervals[1]
             bayes_factor = ",".join(["%.2f" %(max(v, 0)) for v in bf])
@@ -364,50 +373,14 @@ def output_samples_comparison(sample1_dir, sample2_dir, output_dir,
                          gene_info["mRNA_ends"]]
 	output_line = "%s\n" %("\t".join(output_fields))
 	output_file.write(output_line)
-        
-	# Output raw delta posteriors
-	dp_header = "delta_posteriors\n"
-
-        # Move to next batch if needed
-        if file_num % batch_size == 0:
-            curr_batch += 1
-            print "Outputting batch number %d (batch size = %d)..." \
-                  %(curr_batch, batch_size)
-
-            # Make output dir for the current batch
-            batch_dir_name = "batch_%d_%d" %(batch_size, curr_batch)
-            curr_dp_dir = os.path.join(dp_output_dir, batch_dir_name)
-            
-            if not os.path.isdir(curr_dp_dir):
-                print "Making output directory: %s" %(curr_dp_dir)
-                os.makedirs(curr_dp_dir)
-            
-        file_num += 1
-
-        # File name for delta posterior file
-        dp_filename = os.path.join(curr_dp_dir,
-                                   sample1_event_name + '.miso_dp')
-
-        # Output the raw delta posteriors
-	dp_file = open(dp_filename, 'w')
-	dp_file.write(dp_header)
-
-        delta_posteriors = delta_densities['samples1'] - \
-                           delta_densities['samples2']
-
-	for delta_posterior in delta_posteriors:
-            if num_isoforms == 2:
-                delta_posterior = delta_posterior[0:-1]
-            dp_output_line = "%s\n" %(",".join(["%.4f" %(v) for v in delta_posterior]))
-	    dp_file.write(dp_output_line)
-	dp_file.close()
-                                 
 
     print "Compared a total of %d events." %(num_events_compared)
     output_file.close()
-    
 
-def compute_bayes_factor(prior_density, posterior_density, at_point=0, print_bayes=False):
+
+def compute_bayes_factor(prior_density, posterior_density,
+                         at_point=0,
+                         print_bayes=False):
     """
     Compute Bayes factor for given fitted densities.
     """
@@ -432,8 +405,13 @@ def compute_bayes_factor(prior_density, posterior_density, at_point=0, print_bay
     if print_bayes:
 	print "diff_posterior: %.4f" %(diff_posterior)
 	print "bayes_factor: %.2f" %(bayes_factor)
+
+    # Upper bound on Bayes factor
+    if bayes_factor > max_bf:
+        bayes_factor = max_bf
 	
     return bayes_factor, diff_prior, diff_posterior
+
     
 def main():
     pass
