@@ -23,7 +23,6 @@ import misopy.index_gff as index_gff
 from index_gff import is_compressed_index
 
 from misopy.parse_csv import *
-from misopy.json_utils import *
 from misopy.samples_utils import *
 
 import numpy as np
@@ -51,8 +50,10 @@ def compute_gene_psi(gene_ids, gff_index_filename, bam_filename,
     - Optional: Run in paired-end mode. Gives mean and standard deviation
       of fragment length distribution.
     """
-    if not os.path.isdir(output_dir):
+    try:
         os.makedirs(output_dir)
+    except OSError:
+            pass
         
     if not os.path.exists(gff_index_filename):
         print "Error: No GFF %s" %(gff_index_filename)
@@ -178,8 +179,11 @@ def compute_gene_psi(gene_ids, gff_index_filename, bam_filename,
             chrom_dir = os.path.join(output_dir, event_type, gene_obj.chrom)
         else:
             chrom_dir = os.path.join(output_dir, gene_obj.chrom)
-        if not os.path.isdir(chrom_dir):
+
+        try:
             os.makedirs(chrom_dir)
+        except OSError:
+            pass
 
         # Pick .miso output filename based on the pickle filename
         miso_basename = os.path.basename(gff_index_filename)
